var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { connect, createDataItemSigner } from '@permaweb/aoconnect';

import { AO, GATEWAYS } from '../helpers/config';
import { getTxEndpoint } from '../helpers/endpoints';
import { getTagValue, globalLog } from '../helpers/utils';

import { getGQLData } from './gql';
const GATEWAY = GATEWAYS.goldsky;
const GATEWAY_RETRY_COUNT = 1000;
const { result, results, message, spawn, dryrun } = connect({ GATEWAY_URL: `https://${GATEWAY}` });
export function aoSpawn(args) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        const tags = [{ name: 'Authority', value: AO.mu }];
        if (args.tags && args.tags.length > 0)
            args.tags.forEach((tag) => tags.push(tag));
        try {
            const processId = yield spawn({
                module: args.module,
                scheduler: args.scheduler,
                signer: createDataItemSigner(args.wallet),
                tags: tags,
                data: args.data,
            });
            return processId;
        }
        catch (e) {
            throw new Error((_a = e.message) !== null && _a !== void 0 ? _a : 'Error spawning process');
        }
    });
}
export function aoSend(args) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const tags = [{ name: 'Action', value: args.action }];
            if (args.tags)
                tags.push(...args.tags);
            const data = args.useRawData ? args.data : JSON.stringify(args.data);
            const txId = yield message({
                process: args.processId,
                signer: createDataItemSigner(args.wallet),
                tags: tags,
                data: data,
            });
            return txId;
        }
        catch (e) {
            throw new Error(e);
        }
    });
}
export function aoDryRun(args) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        try {
            const tags = [{ name: 'Action', value: args.action }];
            if (args.tags)
                tags.push(...args.tags);
            let dataPayload;
            if (typeof args.data === 'object') {
                dataPayload = JSON.stringify(args.data || {});
            }
            else if (typeof args.data === 'string') {
                try {
                    JSON.parse(args.data);
                }
                catch (e) {
                    console.error(e);
                    throw new Error('Invalid JSON data');
                }
                dataPayload = args.data;
            }
            const response = yield dryrun({
                process: args.processId,
                tags: tags,
                data: dataPayload,
            });
            if (response.Messages && response.Messages.length) {
                if (response.Messages[0].Data) {
                    return JSON.parse(response.Messages[0].Data);
                }
                else {
                    if (response.Messages[0].Tags) {
                        return response.Messages[0].Tags.reduce((acc, item) => {
                            acc[item.name] = item.value;
                            return acc;
                        }, {});
                    }
                }
            }
        }
        catch (e) {
            throw new Error((_a = e.message) !== null && _a !== void 0 ? _a : 'Error dryrunning process');
        }
    });
}
export function aoMessageResult(args) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const { Messages } = yield result({ message: args.messageId, process: args.processId });
            if (Messages && Messages.length) {
                const response = {};
                Messages.forEach((message) => {
                    const action = getTagValue(message.Tags, 'Action') || args.action;
                    let responseData = null;
                    const messageData = message.Data;
                    if (messageData) {
                        try {
                            responseData = JSON.parse(messageData);
                        }
                        catch (_a) {
                            responseData = messageData;
                        }
                    }
                    const responseStatus = getTagValue(message.Tags, 'Status');
                    const responseMessage = getTagValue(message.Tags, 'Message');
                    response[action] = {
                        id: args.messageId,
                        status: responseStatus,
                        message: responseMessage,
                        data: responseData,
                    };
                });
                return response;
            }
            else
                return null;
        }
        catch (e) {
            console.error(e);
        }
    });
}
export function aoMessageResults(args) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const tags = [{ name: 'Action', value: args.action }];
            if (args.tags)
                tags.push(...args.tags);
            yield message({
                process: args.processId,
                signer: createDataItemSigner(args.wallet),
                tags: tags,
                data: JSON.stringify(args.data),
            });
            yield new Promise((resolve) => setTimeout(resolve, 1000));
            const messageResults = yield results({
                process: args.processId,
                sort: 'DESC',
                limit: 100,
            });
            if (messageResults && messageResults.edges && messageResults.edges.length) {
                const response = {};
                for (const result of messageResults.edges) {
                    if (result.node && result.node.Messages && result.node.Messages.length) {
                        const resultSet = [args.action];
                        if (args.responses)
                            resultSet.push(...args.responses);
                        for (const message of result.node.Messages) {
                            const action = getTagValue(message.Tags, 'Action');
                            if (action) {
                                let responseData = null;
                                const messageData = message.Data;
                                if (messageData) {
                                    try {
                                        responseData = JSON.parse(messageData);
                                    }
                                    catch (_a) {
                                        responseData = messageData;
                                    }
                                }
                                const responseStatus = getTagValue(message.Tags, 'Status');
                                const responseMessage = getTagValue(message.Tags, 'Message');
                                if (action === 'Action-Response') {
                                    const responseHandler = getTagValue(message.Tags, 'Handler');
                                    if (args.handler && args.handler === responseHandler) {
                                        response[action] = {
                                            status: responseStatus,
                                            message: responseMessage,
                                            data: responseData,
                                        };
                                    }
                                }
                                else {
                                    if (resultSet.indexOf(action) !== -1) {
                                        response[action] = {
                                            status: responseStatus,
                                            message: responseMessage,
                                            data: responseData,
                                        };
                                    }
                                }
                                if (Object.keys(response).length === resultSet.length)
                                    break;
                            }
                        }
                    }
                }
                return response;
            }
            return null;
        }
        catch (e) {
            console.error(e);
        }
    });
}
export function waitForProcess(processId, _setStatus) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        let retries = 0;
        while (retries < GATEWAY_RETRY_COUNT) {
            yield new Promise((r) => setTimeout(r, 2000));
            const gqlResponse = yield getGQLData({
                gateway: GATEWAY,
                ids: [processId],
            });
            if ((_a = gqlResponse === null || gqlResponse === void 0 ? void 0 : gqlResponse.data) === null || _a === void 0 ? void 0 : _a.length) {
                const foundProcess = gqlResponse.data[0].node.id;
                globalLog(`Fetched transaction: ${foundProcess} (Try ${retries + 1})`);
                return foundProcess;
            }
            else {
                globalLog(`Transaction not found: ${processId} (Try ${retries + 1})`);
                retries++;
            }
        }
        throw new Error(`Process not found, please try again`);
    });
}
export function fetchProcessSrc(txId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const srcFetch = yield fetch(getTxEndpoint(txId));
            return yield srcFetch.text();
        }
        catch (e) {
            throw new Error(e);
        }
    });
}
export function handleProcessEval(args) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        let src = null;
        if (args.evalSrc)
            src = args.evalSrc;
        else if (args.evalTxId)
            src = yield fetchProcessSrc(args.evalTxId);
        if (src) {
            try {
                const evalMessage = yield aoSend({
                    processId: args.processId,
                    wallet: args.wallet,
                    action: 'Eval',
                    data: src,
                    tags: args.evalTags || null,
                    useRawData: true,
                });
                globalLog(`Eval: ${evalMessage}`);
                const evalResult = yield aoMessageResult({
                    processId: args.processId,
                    messageId: evalMessage,
                    action: 'Eval',
                });
                return evalResult;
            }
            catch (e) {
                throw new Error((_a = e.message) !== null && _a !== void 0 ? _a : 'Error sending process eval');
            }
        }
        return null;
    });
}
export function aoCreateProcess(args, statusCB) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a, _b;
        try {
            const spawnArgs = {
                module: args.module || AO.module,
                scheduler: args.scheduler || AO.scheduler,
                wallet: args.wallet,
            };
            if (args.spawnData)
                spawnArgs.data = args.spawnData;
            if (args.spawnTags)
                spawnArgs.tags = args.spawnTags;
            statusCB && statusCB(`Spawning process...`);
            const processId = yield aoSpawn(spawnArgs);
            if (args.evalTxId || args.evalSrc) {
                statusCB && statusCB(`Retrieving process...`);
                yield waitForProcess(processId, statusCB);
                statusCB && statusCB(`Process retrieved!`);
                statusCB && statusCB('Sending eval...');
                try {
                    const evalResult = yield handleProcessEval({
                        processId: processId,
                        evalTxId: args.evalTxId || null,
                        evalSrc: args.evalSrc || null,
                        evalTags: args.evalTags,
                        wallet: args.wallet,
                    });
                    if (evalResult && statusCB)
                        statusCB('Eval complete');
                }
                catch (e) {
                    throw new Error((_a = e.message) !== null && _a !== void 0 ? _a : 'Error creating process');
                }
            }
            return processId;
        }
        catch (e) {
            throw new Error((_b = e.message) !== null && _b !== void 0 ? _b : 'Error creating process');
        }
    });
}
//# sourceMappingURL=ao.js.map