var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import Arweave from 'arweave';

import { TAGS, UPLOAD } from '../helpers/config';
import { checkValidAddress, getBase64Data, getByteSize, getDataURLContentType, globalLog } from '../helpers/utils';
export function createTransaction(args) {
    return __awaiter(this, void 0, void 0, function* () {
        let content = null;
        let contentType = null;
        try {
            if (typeof args.data === 'string' && args.data.startsWith('data:')) {
                content = Buffer.from(getBase64Data(args.data), 'base64');
                contentType = getDataURLContentType(args.data);
            }
        }
        catch (e) {
            throw new Error(e);
        }
        if (content && contentType) {
            const contentSize = getByteSize(content);
            globalLog(`Content upload size: ${contentSize}`);
            if (contentSize < Number(UPLOAD.dispatchUploadSize)) {
                const tx = yield Arweave.init({}).createTransaction({ data: content }, 'use_wallet');
                tx.addTag(TAGS.keys.contentType, contentType);
                if (args.tags && args.tags.length > 0)
                    args.tags.forEach((tag) => tx.addTag(tag.name, tag.value));
                const response = yield global.window.arweaveWallet.dispatch(tx);
                return response.id;
            }
            else {
                throw new Error('Data exceeds max upload limit'); // TODO
            }
        }
        else {
            throw new Error('Error preparing transaction data');
        }
    });
}
export function resolveTransaction(data) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        if (checkValidAddress(data))
            return data;
        else {
            try {
                return yield createTransaction({ data: data });
            }
            catch (e) {
                throw new Error((_a = e.message) !== null && _a !== void 0 ? _a : 'Error resolving transaction');
            }
        }
    });
}
//# sourceMappingURL=arweave.js.map