var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { GATEWAYS } from '../helpers/config';
const CURSORS = {
    p1: 'P1',
    end: 'END',
};
const PAGINATORS = {
    default: 100,
};
export function getGQLData(args) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        const paginator = args.paginator ? args.paginator : PAGINATORS.default;
        let data = [];
        let count = 0;
        let nextCursor = null;
        if (args.ids && !args.ids.length) {
            return { data: data, count: count, nextCursor: nextCursor, previousCursor: null };
        }
        try {
            let queryBody = getQueryBody(args);
            const response = yield getResponse({ gateway: args.gateway, query: getQuery(queryBody) });
            if (response.data.transactions.edges.length) {
                data = [...response.data.transactions.edges];
                count = (_a = response.data.transactions.count) !== null && _a !== void 0 ? _a : 0;
                const lastResults = data.length < paginator || !response.data.transactions.pageInfo.hasNextPage;
                if (lastResults)
                    nextCursor = CURSORS.end;
                else
                    nextCursor = data[data.length - 1].cursor;
                return {
                    data: data,
                    count: count,
                    nextCursor: nextCursor,
                    previousCursor: null,
                };
            }
            else {
                return { data: data, count: count, nextCursor: nextCursor, previousCursor: null };
            }
        }
        catch (e) {
            console.error(e);
            return { data: data, count: count, nextCursor: nextCursor, previousCursor: null };
        }
    });
}
export function getBatchGQLData(args) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        let responseObject = {};
        let queryBody = '';
        for (const [queryKey, baseArgs] of Object.entries(args.entries)) {
            responseObject[queryKey] = { data: [], count: 0, nextCursor: null, previousCursor: null };
            queryBody += getQueryBody(Object.assign(Object.assign({}, baseArgs), { gateway: args.gateway, queryKey: queryKey }));
        }
        try {
            const response = yield getResponse({ gateway: args.gateway, query: getQuery(queryBody) });
            if (response && response.data) {
                for (const queryKey of Object.keys(response.data)) {
                    const paginator = args.entries[queryKey].paginator ? args.entries[queryKey].paginator : PAGINATORS.default;
                    let data = [];
                    let count = 0;
                    let nextCursor = null;
                    if (response.data[queryKey].edges.length) {
                        data = [...response.data[queryKey].edges];
                        count = (_a = response.data[queryKey].count) !== null && _a !== void 0 ? _a : 0;
                        const lastResults = data.length < paginator || !response.data[queryKey].pageInfo.hasNextPage;
                        if (lastResults)
                            nextCursor = CURSORS.end;
                        else
                            nextCursor = data[data.length - 1].cursor;
                        responseObject[queryKey] = {
                            data: [...response.data[queryKey].edges],
                            count: count,
                            nextCursor: nextCursor,
                            previousCursor: null,
                        };
                    }
                }
            }
            return responseObject;
        }
        catch (e) {
            console.error(e);
            return responseObject;
        }
    });
}
function getQuery(body) {
    const query = { query: `query { ${body} }` };
    return JSON.stringify(query);
}
function getQueryBody(args) {
    const paginator = args.paginator ? args.paginator : PAGINATORS.default;
    const ids = args.ids ? JSON.stringify(args.ids) : null;
    let blockFilter = null;
    if (args.minBlock !== undefined && args.minBlock !== null) {
        blockFilter = {};
        blockFilter.min = args.minBlock;
    }
    const blockFilterStr = blockFilter ? JSON.stringify(blockFilter).replace(/"([^"]+)":/g, '$1:') : null;
    const tags = args.tags
        ? JSON.stringify(args.tags)
            .replace(/"(name)":/g, '$1:')
            .replace(/"(values)":/g, '$1:')
            .replace(/"FUZZY_OR"/g, 'FUZZY_OR')
        : null;
    const owners = args.owners ? JSON.stringify(args.owners) : null;
    const cursor = args.cursor && args.cursor !== CURSORS.end ? `"${args.cursor}"` : null;
    let fetchCount = `first: ${paginator}`;
    let txCount = '';
    let nodeFields = `data { size type } owner { address } block { height timestamp }`;
    let order = '';
    switch (args.gateway) {
        case GATEWAYS.arweave:
            break;
        case GATEWAYS.goldsky:
            txCount = `count`;
            break;
    }
    let body = `
		transactions(
				ids: ${ids},
				tags: ${tags},
				${fetchCount}
				owners: ${owners},
				block: ${blockFilterStr},
				after: ${cursor},
				${order}
				
			){
			${txCount}
				pageInfo {
					hasNextPage
				}
				edges {
					cursor
					node {
						id
						tags {
							name 
							value 
						}
						${nodeFields}
					}
				}
		}`;
    if (args.queryKey)
        body = `${args.queryKey}: ${body}`;
    return body;
}
function getResponse(args) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = yield fetch(`https://${args.gateway}/graphql`, {
                method: 'POST',
                headers: { 'Content-Type': 'application/json' },
                body: args.query,
            });
            return yield response.json();
        }
        catch (e) {
            throw e;
        }
    });
}
//# sourceMappingURL=gql.js.map