var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { resolveTransaction } from '../common/arweave';
import { getGQLData } from '../common/gql';
import { GATEWAYS, TAGS } from '../helpers/config';
import { getBootTag, mapFromProcessCase } from '../helpers/utils';

import { createZone, getZone, updateZone } from './zones';
// TODO: Bootloader registry
export function createProfile(args, wallet, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        let profileId = null;
        const tags = [
            { name: TAGS.keys.dataProtocol, value: 'Zone' },
            { name: TAGS.keys.zoneType, value: 'User' }
        ];
        const addImageTag = (imageKey) => __awaiter(this, void 0, void 0, function* () {
            const key = imageKey.toLowerCase();
            if (args[key]) {
                try {
                    const resolvedImage = yield resolveTransaction(args[key]);
                    tags.push(getBootTag(imageKey, resolvedImage));
                }
                catch (e) {
                    if (callback)
                        callback(`Failed to resolve ${imageKey}: ${e.message}`);
                    else
                        console.error(e);
                }
            }
        });
        tags.push(getBootTag('Username', args.username));
        tags.push(getBootTag('DisplayName', args.displayName));
        tags.push(getBootTag('Description', args.description));
        yield Promise.all([addImageTag('Thumbnail'), addImageTag('Banner')]);
        try {
            profileId = yield createZone({ tags: tags }, wallet, callback);
            // if (profileId) {
            // 	globalLog(`Profile ID: ${profileId}`);
            // 	await waitForProcess(profileId);
            // 	const registerId = await aoSend({
            // 		processId: profileId,
            // 		wallet: wallet,
            // 		action: 'Register-Whitelisted-Subscriber',
            // 		tags: [
            // 			{ name: 'Topics', value: JSON.stringify(['Zone-Update']) },
            // 			{ name: 'Subscriber-Process-Id', value: 'Wl7pTf-UEp6SIIu3S5MsTX074Sg8MhCx40NuG_YEhmk' },
            // 		]
            // 	});
            // 	console.log(`Register ID: ${registerId}`);
            // }
        }
        catch (e) {
            throw new Error((_a = e.message) !== null && _a !== void 0 ? _a : 'Error creating profile');
        }
        return profileId;
    });
}
export function updateProfile(args, profileId, wallet, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        if (profileId) {
            let data = {
                Username: args.username,
                DisplayName: args.displayName,
                Description: args.description
            };
            if (args.thumbnail) {
                try {
                    data.Thumbnail = yield resolveTransaction(args.thumbnail);
                }
                catch (e) {
                    if (callback)
                        callback(`Failed to resolve thumbnail: ${e.message}`);
                }
            }
            if (args.banner) {
                try {
                    data.Banner = yield resolveTransaction(args.banner);
                }
                catch (e) {
                    if (callback)
                        callback(`Failed to resolve banner: ${e.message}`);
                }
            }
            try {
                return yield updateZone(data, profileId, wallet);
            }
            catch (e) {
                throw new Error((_a = e.message) !== null && _a !== void 0 ? _a : 'Error creating profile');
            }
        }
        else {
            throw new Error('No profile provided');
        }
    });
}
export function getProfileById(profileId) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c;
        try {
            const zone = yield getZone(profileId);
            return Object.assign(Object.assign({ id: profileId }, mapFromProcessCase((_a = zone.Store) !== null && _a !== void 0 ? _a : {})), { assets: mapFromProcessCase((_b = zone.Assets) !== null && _b !== void 0 ? _b : []) });
        }
        catch (e) {
            throw new Error((_c = e.message) !== null && _c !== void 0 ? _c : 'Error fetching profile');
        }
    });
}
export function getProfileByWalletAddress(walletAddress) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a, _b;
        try {
            const gqlResponse = yield getGQLData({
                gateway: GATEWAYS.arweave,
                tags: [
                    { name: TAGS.keys.dataProtocol, values: ['Zone'] },
                    { name: TAGS.keys.zoneType, values: ['User'] },
                ],
                owners: [walletAddress]
            });
            if (((_a = gqlResponse === null || gqlResponse === void 0 ? void 0 : gqlResponse.data) === null || _a === void 0 ? void 0 : _a.length) > 0) {
                gqlResponse.data.sort((a, b) => {
                    var _a, _b, _c, _d;
                    const timestampA = (_b = (_a = a.node.block) === null || _a === void 0 ? void 0 : _a.timestamp) !== null && _b !== void 0 ? _b : 0;
                    const timestampB = (_d = (_c = b.node.block) === null || _c === void 0 ? void 0 : _c.timestamp) !== null && _d !== void 0 ? _d : 0;
                    return timestampB - timestampA;
                });
                return yield getProfileById(gqlResponse.data[0].node.id);
            }
            return { id: null };
        }
        catch (e) {
            throw new Error((_b = e.message) !== null && _b !== void 0 ? _b : 'Error fetching profile');
        }
    });
}
//# sourceMappingURL=profiles.js.map